/**
 * Copyright (C) 2010 OSGi fuer Praktiker <info@osgi-buch.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package biz.gossipmonger.worker.internal;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Properties;

import org.osgi.service.component.ComponentContext;

import biz.gossipmonger.enunciator.EnunciatorService;
import biz.gossipmonger.worker.WorkerService;

public final class WorkerServiceImpl implements WorkerService {
	private static final String BUNDLE_PROPERTIES_FILE = "worker.properties";
	private static final String PROPERTYNAME_OUTPUT_DIR = "worker.outputDir";
	private static final String DEFAULT_OUTPUT_DIR = "render_output";
	String configuredOutput = DEFAULT_OUTPUT_DIR;
	PdfRenderer renderer = new PdfRenderer();
	EnunciatorService enunciator = null;

	// Component methods

	protected void activate(ComponentContext context) {
		enunciate("activating...");
		configuredOutput = getConfiguredOutput(context);
		enunciate("activated.");
	}

	protected void deactivate(ComponentContext context) {
		enunciate("deactivated.");
	}

	public void setEnunciatorService(EnunciatorService es) {
		if (es != null) {
			enunciator = es;
			enunciate("EnunciatorService set.");
		} else {
			enunciate("setEnunciatorService called with null argument.");
		}
	}

	public void unsetEnunciatorService(EnunciatorService es) {
		enunciate("EnunciatorService unset.");
		enunciator = null;
	}

	// The WorkerService methods

	public File process(File file) {
		return process(file, configuredOutput);
	}

	public File process(File file, String outputDir) {
		enunciate("Render from "+file.getAbsolutePath());
		try {
			return process(new FileInputStream(file), file.getName(), outputDir);
		} catch (FileNotFoundException e) {
			enunciate(file.getAbsolutePath() + " is not there!");
			return null;
		}
	}

	public File process(InputStream is, String inputFilename) {
		return process(is, inputFilename, configuredOutput);
	}

	public File process(InputStream is, String inputFilename, String outputDir) {
		enunciate("Rendering...");
		try {
			File output = renderer.render(is, inputFilename, outputDir);
			enunciate("Rendered to "+output.getAbsolutePath());
			return output;
		} catch (Exception ex) {
			enunciate("Error while rendering: "+ex.getMessage());
			return null;
		}
	}

	// Internal methods

	private void enunciate(String text) {
		if (enunciator != null) {
			enunciator.enunciate("WorkerServiceImpl: " + text);
		} else {
			System.out.println("WorkerServiceImpl(no Enunciator): " + text);
		}
	}

	private String getConfiguredOutput(ComponentContext context) {
		String setBy = "default";
		Properties props = getBundleContainedProperties(context);
		String outputDir = props.getProperty(PROPERTYNAME_OUTPUT_DIR);
		if (outputDir != null && outputDir.isEmpty() == false) {
			setBy = "in-bundle properties";
		} else {
			outputDir = (String) context.getProperties().get(PROPERTYNAME_OUTPUT_DIR);
			if (outputDir != null && outputDir.isEmpty() == false) {
				setBy = "service properties";
			} else {
				outputDir = DEFAULT_OUTPUT_DIR;
			}
		}
		enunciate("Output Directory is set by "+setBy+" to "+outputDir+".");
		return outputDir;
	}
	
	private Properties getBundleContainedProperties(ComponentContext context) {
		Properties properties = new Properties();
		URL url = context.getBundleContext().getBundle().getResource(BUNDLE_PROPERTIES_FILE);
		if (url == null) {
			enunciate(BUNDLE_PROPERTIES_FILE+" file not found in bundle.");
		} else {
			try {
				InputStream is = url.openStream();
				properties.load(is);
				is.close();
			} catch (IOException e) {
				enunciate("Error reading bundle properties file: "+e.getMessage());
			}
		}
		return properties;
	}
}
